/* Progrmme de gestion des TRANSITIONS.

 >>> Avant de téléverser le logiciel sur Arduino passer à false le booléen Aff_Moniteur
 >>> en début de routine void setup() pour annuler les affichages vers le MONITEUR.
 >>> Ne valider cette option que durant le développement.

 >>> DEVELOPPEMENT :
 >>> ---------------
 >>> ATTENTION : Quand la ligne USB est validée pour le Moniteur, le RESET fait disjoncter.
 >>>             ***************************************** *************************************
 >>>             *        POUR RETABLIR L'ENERGIE :      * *  NOTE : Sur RESET la LED T Lect.  *
 >>>             *                                       * *         reste allumée et s'éteint *
 >>>             * Cliquer à la fois sur DISJ et sur MA. * *         durant les impulsions TX. *
 >>>             ***************************************** *************************************

          Par Michel DROUI. (Pseudo Internet : Nulentout.)
          
>>>>>> Si vous rencontrez un problème : michel.droui@laposte.net
          
DIFFERENTS BRANCHEMENTS :
 D0  : RX USB                                  A0 : Pilote le relais R40ST
 D1  : TX USB ou LED RST et "FIN PGM".         A1 : Pilote la LED ER 2
 D2  : Tr1 - R1                                A2 : Isolation C1...C11
 D3  : Tr2 - R2                                A3 : Boque les sorties MATRICE  
 D4  : Tr3 - R3                                A4 : Sortie F de la MATRICE
 D5  : Tr4 - R4                                A5 : Switch (1) (Lecture MATRICE)
 D6  : Tr5 - R5                                A6 : Switch (2) (Sélectionner la ligne MATRICE
 D7  : Tr6 - R6                                A7 : B.P. ER2 et BP Pos n°1.
 D8  : Tr7 - R7
 D9  : Tr8 - R8        
 D10 : Tr9 - R9          
 D11 : Tr10 - R10                             
 D12 : Tr11 - R11
 D13 : Pilote le relais "ALPHA".
 
                                                                PGM               RAM dynamique
 Programme d'exploitation SANS les affichages Moniteur     4908 Octets (16%)      428 Octets (20%) 
 Programme de développement AVEC les affichages Moniteur   5228 Octets (17%)      452 Octets (22%) */

//--------------------- Définition des broches d'E/S ------------------------
#define Active_F_en_Logique 0 // D0 (RX) mémorise F dans la Logique.
#define LED_T_Lect 1 // D1 (TX) Clignote sur RESET.
#define Pilotage_Relais_moteur 14 // Pilote le MOTEUR via A0.
#define LED_ER_2 15 // Pilote la LED via A1.
#define Isolation_C1_C11 16 // Isolation C1...C11 par A2.
#define Bloque_MATRICE 17 // Sortie pour bloquer la MATRICE par A3
#define Sortie_F 18 // Sortie matrice de Fin détectée lue sur A4.
#define Switch_1 19 // Lecture Switch 1 par A5.
#define Switch_2 20 // Lecture Switch 1 par A6.
#define ENTREE_BP 21 // Capter les deux B.P. sur A7.

#define Pilotage_Relais_Alpha 13 // Pilote la coupure Alpha.

//------------------------ Constantes du programme --------------------------

#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation. Ne pas dépasser 50.
#define Delai_voulu 300 // Durée en mS.
#define T_pour_BP_transition 500 // Calibre le délai pour les transitions en mode Manuel.

//------------------------- Variables du programme --------------------------

boolean MOTEUR, Switch_actif, F_actif, Init_Pos_1, Aff_Moniteur, Clignotte_en_boucle_de_base;

byte NB_entrees_a_1;

int Etat_MATRICE, Etat_rotation, Valeurs_lues, Valeur_CAN, Chronometre;

unsigned long Temps_Ecoule_depuis_RESET = 0;
unsigned long Debut_boucle;

//---------------------------------------------------------------------------
void setup(){
  // ATTENTION : Quand on valide TX/RX pour le Moniteur, la sortie D0 qui informe la
  //             logique de la machine qu'un "F" est validé ne fnctionne plus.
  //             Éventuellement enlever le Strap sur la carte ARDUINO quand les
  //             dialogues USB sont validés.

  Aff_Moniteur = false; // Booléen qui active ou suspend les affichages vers le MONITEUR.  
  Clignotte_en_boucle_de_base = false;
  Init_Pos_1 = false; // Signale que l'on est pas en mode initialisation de la position 1.
  if (Aff_Moniteur) {
      Serial.begin(38400); Serial.println(); Serial.println("Version du 25 Aout 2022.");
      Afficher_je_boucle_en_attente();}
  else {//pinMode(Active_F_en_Logique,OUTPUT); digitalWrite(Active_F_en_Logique,LOW);
        pinMode(LED_T_Lect,OUTPUT); digitalWrite(LED_T_Lect,LOW);}
  // ATTENTION : La LED de RX sur NANO est un témoin logique "inversé" qui s'illumine pour
  //             un état "0" et qui s'éteint pour un état "1" sur la sortie D0. (RX)
  //             Donc cette LED sera allumée en permanence sur Arduino NANO.
  pinMode(Pilotage_Relais_moteur,OUTPUT); digitalWrite(Pilotage_Relais_moteur,LOW);
  pinMode(Isolation_C1_C11,INPUT);
  pinMode(Bloque_MATRICE,OUTPUT); digitalWrite(Bloque_MATRICE,LOW);
  pinMode(Pilotage_Relais_Alpha,OUTPUT); digitalWrite(Pilotage_Relais_Alpha,LOW);
  for(byte I=12; I>1; I--) pinMode(I,INPUT);
  pinMode(LED_ER_2,OUTPUT); digitalWrite(LED_ER_2,HIGH); // HIGH : Éteindre la LED.
  //--------- Si sur RESET on clique sur le B.P. INIT. Pos.1 la boucle de base ----
  //-------- void Looop() clignetera sur T Lect pour aide au développement. -------
  Valeur_CAN = Mesurer_ENTREES_BP();
  if (Valeur_CAN > 255 && Valeur_CAN < 766) {
      while (Mesurer_ENTREES_BP() < 850); // Ne rien faire pour attendre le relâcher du B.P.
      Clignotte_en_boucle_de_base = true;}
  //-------------- Faire clignoter la LED T Lect. durant 5 secondes. --------------
  if (Aff_Moniteur) Serial.println();
  // Ne clignote que si Aff_Moniteur = false.
  else {for (byte I=0; I < 51; I++) {
             digitalWrite(LED_T_Lect,!digitalRead(LED_T_Lect)); delay(100);} // 100 en standard.
             digitalWrite(LED_T_Lect,LOW);}}

void loop(){
  Debut_boucle = millis(); // Initialisation du compteur pour mesurer la durée de boucle.
  if (Clignotte_en_boucle_de_base) {
      digitalWrite(LED_T_Lect,HIGH); delay(1); digitalWrite(LED_T_Lect,LOW);delay(100);}
  //------------------- Traite la transition manuelle -----------------------
  Teste_le_BP_ER2();
  if (Chronometre > T_pour_BP_transition) {
      if (Aff_Moniteur) {Serial.print("Chronometre = "); Serial.println(Chronometre);}
      if (Aff_Moniteur) {Serial.println("TRANSITIONS en mode Manuel.");}
      Init_Pos_1 = true; // Ne pas tenir compte du Switch 2 pour traiter.
      Etat_MATRICE = 1; Traiter_la_TRANSITION(); Traite_manuellement_les_transitions();}
  //---------------------- Traite le B.P. init. Pos. 1 ----------------------
  Valeur_CAN = Mesurer_ENTREES_BP();
  if (Valeur_CAN > 255 && Valeur_CAN < 766) {
       digitalWrite(LED_T_Lect,HIGH); // Allume T Lect durant l'appui sur le B.P.
       while (Mesurer_ENTREES_BP() < 850); // Ne rien faire pour attendre le relâcher du B.P.
       digitalWrite(LED_T_Lect,LOW);
       Replacer_sur_la_transition_1(); Afficher_duree_boucle_et_je_boucle_en_attente();}
  //----------------------- Traite le capteur Switch 1 ----------------------
  else {Preparer_une_transition();
       if (Switch_actif) {
        //------------ Si Transition programmée, la traiter -------------
        if (NB_entrees_a_1 > 0) {
            if (NB_entrees_a_1 > 1) Traiter_ER2();
            else Traiter_la_TRANSITION();}
        else {if (Aff_Moniteur) Serial.println("Pas de transition a traiter.");}   
        //------ Attendre que l'opérateur clique sur le B.P. INIT -------
        Teste_F_MATRICE();
        while (F_actif) ;} // Ne rien faire.   
        Afficher_duree_boucle_et_je_boucle_en_attente();}}

//------------------------- Procédures de servitude -------------------------

void Jalon_clignottant(byte T_pulse, byte NB_pulses) {
     // Cette procédure disponible pour le développement sert à repérer le passage
     // dans certaines lignes critiques du PGM quand le Moniteur n'est pas utilisé.
     //-----------------------------------------------------------------------
     // T_pulse : Durée de l'impulsion lumineuse et de l'interpulsations en mS.
     // NB_pulses : Nombre de répétitions d'une pulse et de l'intervalle.
     // Valeurs utiles pour T_pulse : 20 mini, 25 et 35.
     // NB_pulses = Temps total / (2 * T_pulse).
     // Exemple : T_pulse = 25 et Temps total = 2000mS.
     //           NB_pulses = 2000 / (2 * 25) = 40.
     for (byte Pulse = 0; Pulse < NB_pulses; Pulse ++) {
          digitalWrite(LED_T_Lect,HIGH); delay(T_pulse);
          digitalWrite(LED_T_Lect,LOW); delay(T_pulse);}}

void Teste_le_BP_ER2() {
  Chronometre = 0;
  while (Mesurer_ENTREES_BP() < 255) {
         Chronometre++;
         if (Chronometre > T_pour_BP_transition) digitalWrite(LED_ER_2,LOW);} // Allume ER2.
  digitalWrite(LED_ER_2,HIGH);} // Eteint la LED ER2 : La logique de la LED est inversée.

void Traite_manuellement_les_transitions() {
  boolean Fini = false;
  while (!Fini){
        //--------------- Vérifie si le B.P. ER2 est cliqué ---------------
        while (Mesurer_ENTREES_BP() > 250) { // Attendre une activation du B.P. ER2.
               digitalWrite(LED_ER_2,LOW); delay(20); digitalWrite(LED_ER_2,HIGH); delay(50);}
        //---------------- Vérifie si le BP ER2 est cliqué ----------------
        Teste_le_BP_ER2(); Fini = Chronometre > T_pour_BP_transition;
        //--------- Si NON Fini traiter la transition suivante --------
        if (!Fini) {Etat_MATRICE = Etat_MATRICE << 1; if (Etat_MATRICE == 2048) Etat_MATRICE = 1;
                   Traiter_la_TRANSITION();}}
  //---------------- Ramène en Tr1 et retour en boucle de base -----------
  Etat_MATRICE = 1; Traiter_la_TRANSITION();
  Init_Pos_1 = false;}

void Afficher_la_duree_de_boucle() {
   Serial.println(); Serial.print("Duree boucle = ");
   Serial.print(millis() - Debut_boucle); Serial.println("mS.");}
      
void Afficher_je_boucle_en_attente() {
  Serial.println(); Serial.println("Je boucle en attente.");}

void Afficher_duree_boucle_et_je_boucle_en_attente() {
  if (Aff_Moniteur) {Afficher_la_duree_de_boucle(); Afficher_je_boucle_en_attente();}}
  
void Afficher_Etat_Matrice() {
  byte Compteur = 11; byte BIT;
  int Clone_Etat_MATRICE = Etat_MATRICE;
  int EMPREINTE = Etat_MATRICE;
  Serial.println(); Serial.print("Etat MATRICE = ");
  while (Compteur > 0) {
         EMPREINTE = Clone_Etat_MATRICE & 1024;
         if(EMPREINTE == 1024) Serial.print("1"); else Serial.print("0");
         Clone_Etat_MATRICE = Clone_Etat_MATRICE << 1;
         if (Compteur == 11) Serial.print("-"); if (Compteur == 6) Serial.print("-");
         Compteur--;}
  Serial.println();}
      
void Afficher_Etat_Rotation() {
  byte Compteur = 11; byte BIT;
  int Clone_Etat_MATRICE = Etat_rotation;
  int EMPREINTE = Etat_rotation;
  if (Aff_Moniteur) {
      Serial.println(); Serial.print("Etat Switchs = "); 
      while (Compteur > 0) {
             EMPREINTE = Clone_Etat_MATRICE & 1024;
             if(EMPREINTE == 1024) Serial.print("1"); else Serial.print("0");
             Clone_Etat_MATRICE = Clone_Etat_MATRICE << 1;
             if (Compteur == 11) Serial.print("-"); if (Compteur == 6) Serial.print("-");
             Compteur--;}
      Serial.println();}}

int Mesurer_ENTREES_BP() { // Valeurs lues : 1022 / 512 / 0.
  Valeurs_lues = 0;
  for (int I = 0; I < 25 ; I++) // Cumuler N mesures. 
      {Valeurs_lues = Valeurs_lues + (analogRead(ENTREE_BP)); }  
  Valeurs_lues = int (Valeurs_lues / 25); return Valeurs_lues;}

void Tester_un_Switch(byte SWITCH) { // La lecture sur A0 donne 0 ou 1023.
  byte COMPTEUR; unsigned int Total_lecture = 0;
  Switch_actif = false;
  //-------------- Tester si le SWITCH est acivé --------------
  for (byte I=0; I < NB_tests_antirebonds; I++) Total_lecture = Total_lecture + analogRead(SWITCH);
  Total_lecture = int(Total_lecture / NB_tests_antirebonds);
  if (Total_lecture < 512) Switch_actif = true;
  //-----------------------------------------------------------
  if (Switch_actif) {
      //------------ Attendre la stabilisation à GND ----------
      COMPTEUR = 0;
      while (COMPTEUR < NB_tests_antirebonds) {
             COMPTEUR++; if (analogRead(SWITCH) > 512) COMPTEUR = 0;}
      //---------------- Attendre le retour à "1" -------------
     if(SWITCH != Switch_2) {// Ne pas attendre le relâcher pour le Switch 2.     
         COMPTEUR = 0;
         while (COMPTEUR < NB_tests_antirebonds) {
                COMPTEUR++; if (analogRead(SWITCH) < 512) COMPTEUR = 0;}}}}
           
void Lire_etat_rotation() {
  int BIT;
  Etat_rotation = 0;
  for (byte I=12; I > 2; I--)
      {BIT = digitalRead(I); Etat_rotation = Etat_rotation | BIT;
       Etat_rotation = Etat_rotation << 1;}
  BIT = digitalRead(2); Etat_rotation = Etat_rotation | BIT;
  Etat_rotation = Etat_rotation ^ 2047; // Inverse tous les bits de l'état capté.
  } // Ici possibilité d'ajouter éventuellement l'appel à Afficher_Etat_Rotation().

void Preparer_une_transition() {
     Tester_un_Switch(Switch_1); // Test du Switch de déclenchement d'une TRANSITION.
     if (Switch_actif) {// Lecture 1043 non actif et 0 si Actif.
         Lire_les_sorties_MATRICE(); Compte_NB_etats_1();
         if (Aff_Moniteur) Afficher_Etat_Matrice();}}
                
void Traiter_la_TRANSITION() { // Déclenchement par le Switch 2 ou en FIN de PGM.
  byte TrN = 1;
  int Clone_Etat_MATRICE = Etat_MATRICE;
  //------------ Affiche le n° de TRN validé par la MATRICE ------------
  while ((Clone_Etat_MATRICE & 1) == 0) {TrN++; Clone_Etat_MATRICE = Clone_Etat_MATRICE >> 1;}
  if (Aff_Moniteur) {Serial.print("Je traite Tr"); Serial.print(TrN); Serial.println(".");}
  //---------------- Traitement la rotation pour TrN -------------------
  if(!Init_Pos_1) // Ne pas tenir compte de Switch 2 si mode Init. pos. 1.
    {Switch_actif = false;
     while (Switch_actif == false) Tester_un_Switch(Switch_2);}
  //--------------------------------------------------------------------
  if (Aff_Moniteur) {Serial.print("Indexation de "); Serial.print(TrN); Serial.println(".");}
  pinMode(Isolation_C1_C11,OUTPUT); digitalWrite(Isolation_C1_C11,LOW); // Le commun est à "0";
  digitalWrite(Bloque_MATRICE,HIGH); // Force les 11 entrées à "1".
  Lire_etat_rotation(); // Si déjà sur la position n'activera pas le relais MOTEUR :
  while (Etat_rotation != Etat_MATRICE) {
         Lire_etat_rotation(); digitalWrite(Pilotage_Relais_moteur,HIGH);}
  //-------------------- Reconfigure en mode veille --------------------
  digitalWrite(Pilotage_Relais_moteur,LOW);
  pinMode(Isolation_C1_C11,INPUT); digitalWrite(Bloque_MATRICE,LOW);
  delay(500);} // Délai impératif pour que l'Horloge revienne sur ARRET.

void Replacer_sur_la_transition_1() {
  if (Aff_Moniteur) Serial.println("Init. Pos. 1.");
  Init_Pos_1 = true; Etat_MATRICE = 1; Traiter_la_TRANSITION(); Init_Pos_1 = false;}

void Traiter_ER2() {
  if (Aff_Moniteur) {Serial.println("Je traite ER2."); Serial.println();}
  digitalWrite(Pilotage_Relais_Alpha,HIGH); // Coupure de la ligne "Alpha".
  while (Mesurer_ENTREES_BP() > 255)  {
        //================= Chronométrage pour le délai écoulé ================
        if (millis() - Temps_Ecoule_depuis_RESET > Delai_voulu)  {Temps_Ecoule_depuis_RESET = millis();
        //----------------- Inverser l'état de la LED. -----------------
        digitalWrite(LED_ER_2,!digitalRead(LED_ER_2));}}
        //---------------------------------------------------------------------
  while (Mesurer_ENTREES_BP() < 255){}; // Attendre et ne rien faire tant que le B.P. est actif.
  digitalWrite(LED_ER_2,HIGH);
  digitalWrite(Pilotage_Relais_Alpha,LOW);} // Rétablir la ligne "Alpha".

void Compte_NB_etats_1() {
  int Clone_etat_MATRICE = Etat_MATRICE; 
  NB_entrees_a_1 = 0;
  for (byte I=0; I < 11; I++) {
       NB_entrees_a_1 = NB_entrees_a_1 + (Clone_etat_MATRICE & 1);
       Clone_etat_MATRICE = Clone_etat_MATRICE >> 1;}}

void Teste_F_MATRICE() {
  F_actif = (analogRead(Sortie_F) > 255);
  if (Aff_Moniteur) {Serial.print("F actif = "); Serial.println(F_actif);}}

void Lire_les_sorties_MATRICE() {
  int BIT;
  digitalWrite(Bloque_MATRICE,LOW); // A3 = 0 n'intervient plus sur les diodes.
  pinMode(Isolation_C1_C11,INPUT); // La broche analogique A2 devient une entrée.
  Etat_MATRICE = 0;
  for (byte I=12; I > 2; I--)
      {BIT = digitalRead(I); Etat_MATRICE = Etat_MATRICE | BIT; Etat_MATRICE = Etat_MATRICE << 1;}
      BIT = digitalRead(2); Etat_MATRICE = Etat_MATRICE | BIT;}






